local sbyte=string.byte

local function size(us)
   return us:sub(1,1):byte()*2048383+us:sub(2,2):byte()*16129+us:sub(3,3):byte(
          )*127+us:sub(4,4):byte()
end

local function header(f)
   local h=f:read(10)
   if h and h:sub(1,3)=='ID3' then
      local v=h:sub(4,4):byte()
      if v~=3 and v~=4 then return nil end
      if h:sub(6,6)~='\0' then return nil end
      return v,size(h:sub(7,10))
   end
end

local function frame(f)
   local h=f:read(10)
   if h then
      local id,siz,fl1,fl2,data=
         h:sub(1,4),size(h:sub(5,8)),h:sub(9,9):byte(),h:sub(10,10):byte()
      if siz>0 then data=f:read(siz) end
      if data then return id,data end
   end
end

local function extractId3Tag(io,dn,fn,authorize)
   local id3=nil
   if dn and fn then
      fn=dn..fn
      authorize(_ENV,fn,"GET")
      local st = io:stat(fn)
      local fp,e = io:open(fn,"rb")
      if st and fp then
         local tagv,taglen=header(fp)
         if tagv then
            local len=10
            local tags={}
            while true do
               local id,data=frame(fp)
               if not id then break end
               len=len+10+#data
               if len>taglen or id==nil then break end
               if (id:sub(1,1)=='T' or id=='COMM') and data:sub(1,1)=='\0' then
                  if data:sub(-1,-1)=='\0' then data=data:sub(2,-2)
                  else data=data:sub(2) end
               end
               if id=='TXXX' then
                  if not tags.TXXX then tags.TXXX={} end
                  tags.TXXX[#tags.TXXX+1]=data
               elseif id=='COMM' then
                  if not tags.COMM then tags.COMM={} end
                  tags.COMM[#tags.COMM+1]=data:sub(5,-1)
               else tags[id]=data end
            end
            if tags.TIT2 then
               id3={
                  artist=tags.TPE1,title=tags.TIT2, album=tags.TALB,
                  comment=tags.COMM
               }
            end
         else
            fp:seek(st.size-128)
            local tag = fp:read(3)
            if tag == "TAG" then
               id3={}
               id3.title=fp:read(30)
               id3.artist=fp:read(30)
               id3.album=fp:read(30)
               id3.year=fp:read(4)
               id3.comment=fp:read(30)
               if sbyte(id3.comment, 29) == 0 then
                  id3.track = sbyte(id3.comment, 30)
               end
               id3.genre=fp:read(1)
            end
         end
         fp:close()
      end
   end
   return id3
end

local fsqviewBusy=false


local function sendJpg(response,im)
   local data=im:jpegStr(80)
   if data then
      response:setcontenttype"image/jpeg"
      response:setcontentlength(#data)
      response:send(data)
      return
   end
   response:senderror(503)
end


local function fsqview(_ENV,rel,isPost,io,authorize)
   if isPost then
      response:setcontenttype("text/plain")
      response:write('ok');
   else
      authorize(_ENV,rel,"GET")
      collectgarbage()
      local name=request:data("name")
      if name then
         local path=io:realpath(rel..name)
         if path then
            local cnt=0
            while fsqviewBusy do
               cnt=cnt+1
               if(cnt > 25) then
                  response:senderror(503)
                  return
               end
               ba.sleep(100)
            end
            fsqviewBusy=true
            local im = gd.createFromJpeg(path)
            if im then
               local xw,xh
               local maxw,maxh=360,270
               local w,h = im:sizeXY()
               if w > maxw then
                  diff = w/maxw
                  xh = h/diff
                  if xh > maxh then
                     diff = h/maxh
                  end
               elseif h > maxh then
                  diff = h/maxh
               end
               if diff then
                  xw = w/diff
                  xh = h/diff
               else
                  xw = w
                  xh = h
               end
               if xw < w or xh < h then
                  local im2 = gd.createTrueColor(xw,xh)
                  if im2 then
                     gd.copyResized(im2,im,0,0,0,0,xw,xh,w,h)
                     im=im2
                  end
               end
               fsqviewBusy=false
               sendJpg(response,im)
               im,data=nil,nil
               collectgarbage()
               response:abort()
            end
            fsqviewBusy=false
         end
      end
      sendJpg(response,gd.create(1,1))
      response:abort()
   end
end

local function email(_ENV,rel,io,authorize)
   local files={}
   for k,v in request:datapairs() do
      if k=="n" then table.insert(files,v) end
   end
   if #files > 0 then
      for _,name in pairs(files) do
         authorize(_ENV,rel..name,"GET")
      end
      local s = request:session()
      local e = s.email or {}
      e.path=rel
      e.files=files
      e.io=io
      s.email=e
      response:sendredirect"/rtl/protected/mail/send/"
   end
   response:senderror(404)
end


local function create(...)
   local fs=ba.create.wfs(...)
   local authorize,io
   local t = fs:configure{
      tmo=bd.cfg.load().v.davtmo,
      logouturi="/rtl/logout.lsp",
      helpuri="/rtl/wfshelp/",
      ctxmuri="/rtl/wfm-menu/ctxmenu.js",
      dircmd={
         qview=function(_ENV,rel,isPost)
            fsqview(_ENV,rel,isPost,io,authorize)
         end,
         id3tag=function(_ENV,rel)
            authorize(_ENV,rel,"GET")
            local id3=extractId3Tag(io,rel,request:data"name",authorize)
            if id3 then response:json(id3) end
            response:senderror(404)
         end,
         email=function(_ENV,rel) email(_ENV,rel,io,authorize) end,
      },
   }
   authorize=t.authorize
   io=t.io
   return fs
end

return create

