<?lsp

local trim=bd.rtrim
 

local lspInfo="This is a dynamic application. The application contains Lua Server Pages (LSP), which is executed in the FuguHub Application Server."

statusCodes={
MemError="MemError",
HTML="This is a static HTML application. The application is passive on the server and no code is executed by the FuguHub Web Operating System.",
LSP=lspInfo,
LSPVM=lspInfo.."This LSP application runs in a separate Lua Virtual Machine.",
Unknown="The application type is unknown. The application is missing the \".appinfo\" file.",
InvalidInfo="The \".appinfo\" file is corrupt.",
NotFound="The application was not found",
IoError="Some kind of I/O error occurred while loading the application.",
ZipError="The ZIP file for this application is corrupt.",
LspLaunchError="This is a dynamic application, but the application failed to load due to an error in the application code."
}



local data = request:data()
local selApp = data.n and bd.appmgr.list[data.n] or nil
if selApp then
   if data.action then
      if data.action == "start" then
         app = bd.appmgr.load(selApp.name, selApp.fn, selApp.prio)
         if not app or not app.running then
            response:forward(".error.lsp");
            return;
         end
      elseif data.action == "stop" then
         bd.appmgr.stop(selApp.name)
      elseif data.action == "remove" then
         bd.appmgr.remove(selApp.name)
         response:sendredirect("./")
         return
      end
      selApp = bd.appmgr.list[data.n]
   elseif data.prio then
      data.prio = tonumber(data.prio)
      data.domain=trim(data.domain)
      if data.prio ~= selApp.prio or
         data.domain ~= selApp.domain then
         bd.appmgr.stop(selApp.name)
         selApp = bd.appmgr.load(selApp.name,selApp.fn,data.prio,
                                 data.domain)
         bd.config.save()
      end
   end
elseif data.n then
   response:sendredirect("./")
   return
end

selApp = selApp or bd.appmgr.list[next(bd.appmgr.list)]
if not selApp then
   response:forward(".noapp.lsp");
   return;
end

--Load and parse the application config file.
local info
local infostr = bd.info(selApp.name); 
if infostr then
   info = json.decode(infostr)
end
info = info or {}
info.copyright = info.copyright or "Unknown"
info.description = info.description or "Unknown"


local startAction = selApp.running and "stop" or "start"

-- The extra header we need ".header" to add.
headerdata=[[
<link href="style.css" rel="stylesheet" type="text/css" />
<link href="../style.css" rel="stylesheet" type="text/css" />
<style type="text/css" media="screen">
#ctrlBar{margin-bottom: 2em;padding-bottom: 1.3em;font-weight: bold;font-size:120%}
#ctrlBar a{display: block; width: 8em; float: left;margin:0;margin-right: 3px;text-align:center}
#ctrlBar a:hover{background: black;color:white}
</style>
]]

title="Menu: Administrator Panel: Applications"
response:include"../.header.lsp"
emitHelpBut("appmgr", true)
?>
<div id="appmgr">
<h1 style="maring-top:0">Application Name: <label id='appName'><?lsp=selApp.name?></label></h1>

  <div id="ctrlBar">
    <a href='app.lsp?n=<?lsp=selApp.name.."&action="..startAction?>'><?lsp=startAction?></a>
    <?lsp if not selApp.running then
                print("<a href='app.lsp?n="..selApp.name.."&action=remove'>remove</a>")
              end
    ?>
  </div>
  <dl>
    <?lsp
    if info.description == "Unknown" then
    ?>&nbsp;
    <?lsp
      else
    ?>
    <dt>Description:</dt>
    <dd><?lsp=info.description?></dd>
    <?lsp
    end
    ?>
    <?lsp
    if info.link then
    ?>
    <dt>Home page: </dt>
    <dd>
    <?lsp
      response:write("<a href='",info.link,"' target='_blank'>", info.link, "</a>");
    end
    ?>
    </dd>
    <dt>Application information:</dt>
    <dd>
        File name: <?lsp=selApp.fn?><br/>
        <?lsp
            if selApp.status then
               print("Root application:",
                     info.rootapp and "yes" or "no", "<br/>")
               if info.rootapp == false and selApp.running then
                  response:write("URL: <a href=/",selApp.name,
                                 " target='_blank'>http://",
                                 request:header("host"),"/",selApp.name,"</a>")
               end
            end
        ?>
    </dd>
    <?lsp if selApp.status then ?>
    <dt>Status: </dt>
    <dd><b><?lsp=selApp.status?></b>: <p class="desc"><?lsp=statusCodes[selApp.status]?><p></dd>
    <?lsp 
     if info.copyright == "Unknown" then
    ?>&nbsp;
    <?lsp
     else
    ?>
    <dt>Copyright: </dt>
    <dd><?lsp=info.copyright?></dd>
    <?lsp
      end
    ?>
    <?lsp
      if info.rootapp == true then
    ?>
    <dt>Root application settings: </dt>
    <dd>
    <form method="post">
    <table>
      <tr>
        <td>Priority:</td>
        <td><select name="prio" style="width: 5em">
        <?lsp
         local sel = selApp.prio ~= nil and selApp.prio or 0
         for i=-1,2 do
            print(string.format('<option value="%d" %s>%d</option>',
                                i,i == sel and 'selected="true"' or '',i))
         end
         local domain = selApp.domain and selApp.domain or ""
       ?>
        </select></td>
      </tr>
      <tr>
        <td>Domain&nbsp;name:</td>
        <td><input type="text" size="40" name="domain" value="<?lsp=domain?>" title="Name based virtual host domain name"/></td>
      </tr>
    </table>
    <input type="submit" value="Save" />
    </form>
    </dd>
    <?lsp
      end
      end
    ?>
  </dl>
</div>
<?lsp response:include"../.footer.lsp" ?>
