<?lsp

local tinsert=table.insert
local fmt=string.format
local list2tab = bd.list2tab
local tab2list = bd.tab2list

-- Collect all url encoded data.
-- "croles" and "aroles" are from the select:option multisets
-- croles: Constraint roles, aroles: All roles
local data={croles={},aroles={}}
do
   local action={
      croles=function(val) tinsert(data.croles, val) end,
      aroles=function(val) tinsert(data.aroles, val) end,
      constr=function() end
   }
   for key,val in request:datapairs() do
      val = val:gsub("^%s*(.-)%s*$", "%1") -- trim
      if #val ~= 0 then
         if not data[key] then
            data[key]=val:gsub("[<>]","")
         else
            local action=action[key]
            if not action then
               response:senderror(403)
               return
            end
            action(val)
         end
      end
   end
end

local function sendredirect(key,val)
   local url=request:url()
   if key then
      response:sendredirect(fmt("%s?%s=%s",url, key, val and ba.urlencode(val) or ""))
   else
      response:sendredirect(url)
   end
end

local function sendredirect2constr()
   if data.constr then
      sendredirect("constr", data.constr)
   else
      sendredirect()
   end
end

local function emitCheckbox(name, isChecked)
   response:write('<input type="checkbox" name="',name,'" id="',name,'"',
                  isChecked and " checked" or "",'/>')
end

local function emitOptions(list, excludeList, selname)
   local ex=list2tab(excludeList)
   table.sort(list)
   for _,v in ipairs(list) do
      if not ex[v] then
         response:write('<option ',selname == v and 'selected="selected" ' or '',
                        'value="',ba.urlencode(v),'">',v,'</option>')
      end
   end
end

local constrs=bd.db.tuncnstr.load().v -- All constraints

 -- Handle Constraint Submit Button first, if clicked
if data.constrsub then
   if data.constr then -- name
      if data.constrsub == "Delete" then
         constrs[data.constr]=nil
         bd.db.tuncnstr.save({v=constrs})
      else
         sendredirect2constr()
         return
      end
   end
   sendredirect()
   return
end

-- Extract data required for the various page click and submit actions
local aroles=bd.db.roles.load().v -- All roles
local constrnames=tab2list(constrs) -- Create a list of constr. names
local selconstr=constrs[data.constr] -- The selected constraint, if any
local session = request:session(true)

local sconstr = session.tconstr -- The session (saved) constraint, if any
if sconstr then
   if sconstr.name~=data.constr then
      if selconstr then
         sconstr=selconstr
         if sconstr then
            sconstr.name=data.constr
         end
      end
   end
end


local notsaved
if sconstr then
   notsaved = true
else
   notsaved = false
   if selconstr then
      sconstr=selconstr
   else -- Create default vals
      sconstr={
         roles={"user"},
         urls={},
         methods={"GET"}
      }
   end
end

local disabled = sconstr.methods[1] ~= "GET"

session.tconstr=sconstr
if data.constr then
   sconstr.name = data.constr
elseif not sconstr.name then
   sconstr.name = ""
end

-- Take care of all submit buttons, except the Constraint Submit Button
if request:method() == "POST" then
   if data.disable then
      sconstr.methods={"DISABLED"}
   else
      sconstr.methods={"GET"}
   end
   if data.pathsub then
      if data.pathsub == "Delete" then
         if data.pathlist then
            local u=list2tab(sconstr.urls)
            u[data.pathlist]=nil
            sconstr.urls=tab2list(u)
         end
      else
         -- Strip invalid data. Idiot proof.
         local h,p
         if data.host then h=data.host:gsub("/*%**$","") end
         if data.port then p=data.port:gsub("/*%**$","") end
         if h and p and #p > 0 then
            p=tonumber(p)
            if p ~= nil and p > 0 and p < 65535 then
               tinsert(sconstr.urls,fmt("%s/%u/*",h,p))
            end
         elseif h then
            tinsert(sconstr.urls,fmt("%s/*",h))
         else
            tinsert(sconstr.urls,fmt("/*",h))
         end
      end
   elseif data.delrolesub then
      local r = list2tab(sconstr.roles)
      for _,v in ipairs(data.croles) do
         r[v]=nil
      end
      sconstr.roles = tab2list(r)
   elseif data.addrolesub then
      local r = list2tab(sconstr.roles)
      for _,v in ipairs(data.aroles) do
         r[v]=true
      end
      sconstr.roles = tab2list(r)
   elseif data.savesub then
      if data.savesub == "Clear" then
         session.tconstr=nil
         sendredirect()
         return
      elseif data.constr then
         constrs[data.constr]=sconstr
         session.tconstr=nil
         bd.db.tuncnstr.save({v=constrs})
         sendredirect2constr()
         return
      end
   end
   if selconstr then
      sendredirect2constr()
   else
      sendredirect()
   end
   return
end

 
headerdata=[[
<link href="style.css" rel="stylesheet" type="text/css" />
<style>
.talignr {text-align:right;}
select,textarea {width:130px;margin:3px}
fieldset { float:left;}
table, tr, td {margin:0;padding:0}
td { vertical-align: top; }
.hiddenc {display:none}
#advancedaccess, #basicaccess { width:150px; }
</style>
<script type="text/javascript">
$(function() {
   $("#gotoconstr").remove();
   $("#constr").mouseup(function() {
      setTimeout(function() {
         window.location = window.location.pathname+"?constr="+escape($("#constr option:selected").text())
      }, 100);
   });
   $("#commonports").show().change(function () {
      $("#port").attr("value",$("#commonports option:selected").val());
   });
});
</script>
]]
 
title="Menu: Administrator Panel: Tunnel Constraints"
response:include".header.lsp"

?>

<div id="panel">
<?lsp emitHelpBut("tunnelconstr") ?>
<form method="POST">
<p><b>HTTPS Tunnel Constraint Name:</b>
<?lsp= selconstr and data.constr or fmt('<input type="text" name="constr" value="%s"/>',sconstr.name) ?>
</p>


<table><tr>


<td>
<fieldset>
  <legend><b> Constraints </b></legend>
    <select id="constr" name="constr" size="30">
      <?lsp emitOptions(constrnames,nil,data.constr) ?>
    </select>
    <br/>
    <input id="gotoconstr" type="submit" name="constrsub" value="Goto" />
    <input id="deleteconstr" type="submit" name="constrsub" value="Delete" <?lsp=selconstr and '' or ' disabled="disabled"'?>/>
</fieldset>
</td>



<td>

<table>
<tr><td colspan="2">
  <fieldset style="width:430px;margin-bottom:10px">
    <legend><b> Host / Port </b></legend>
      <select name="pathlist" size="11" style="width:95%">
      <?lsp emitOptions(sconstr.urls) ?>
      </select>
      <br/>
        <input style="float:right;margin-left:20px;" type="submit" id="pathdel" name="pathsub" value="Delete" />
  </fieldset>
</td></tr>

<tr>
<td>

  <table>
    <tr><td>
    <fieldset>
      <legend><b> Add Host / Port </b></legend>
        <table>
          <tr>
            <th>Host</th><th>Port</th>
          </tr>
          <tr>
            <td>
              <input type="text" id="host" name="host" value=""/>
            </td>
            <td>
              <input style="width:30px" type="text" id="port" name="port" value=""/>
            </td>
          </tr>
          <tr>
            <td>
               <select class="hiddenc" id="commonports" name="commonports" size="1">
               <?lsp
                 local portnames={"Common Ports","All Ports","TELNET","HTTP","FTP","SMTP","POP3","VNC","SVN"}
                 local ports={'','*',"23","80","21","25","110","5900","3690"}
                 for i,v in ipairs(portnames) do
                    response:write('<option value="',ports[i],'">',v,'</option>')
                 end
              ?>
            </select>
            </td>
            <td><input  style="float:right" type="submit" id="pathsub" name="pathsub" value="Add" /></td>
          </tr>
        </table>
    </fieldset>
    </td></tr>
    <tr><td>
    <fieldset style="margin-top:20px">
      <legend><b> Enable /Disable Constraint </b></legend>
      <b>Disable:</b> <?lsp emitCheckbox("disable",disabled) ?>
    </fieldset>
    </td></tr>
  </table>


</td>
<td>
    <table style="float:right"><tr><td>
    <fieldset >
      <legend><b> Constraint's Roles </b></legend>
      <select <?lsp=#sconstr.roles==0 and 'id="superuser"' or ''?> name="croles" multiple="multiple" size="12">
        <?lsp emitOptions(sconstr.roles) ?>
      </select>
    </fieldset>
    </td></tr><tr><td>
    <input type="submit" name="delrolesub" value="Delete Roles" />
    </td></tr></table>
</td>
</tr>

</table>


</td>


<td>
<table><tr><td>
<fieldset><legend><b> All Roles </b></legend>
   <select name="aroles" multiple="multiple" size="30">
   <?lsp emitOptions(aroles,sconstr.roles) ?>
</select></fieldset>
</td></tr><tr><td>
<input type="submit" name="addrolesub" value="Add Roles" />
</td></tr></table>
</td>


</tr></table>

<hr/>
<input type="submit" name="savesub" value="Save" />
<?lsp if notsaved then ?>
<input type="submit" name="savesub" value="Clear" />
<?lsp end ?>
</form>
</div>

<?lsp response:include".footer.lsp" ?>
