<?lsp

if request:data"addusers" then
   response:sendredirect"setuser.lsp"
   return
end

headerdata=[[
<script type="text/javascript" src="setuser.js"></script>
<style type="text/css">
#UserInfo img { padding-left:2em; }
#UserInfo { margin-left: auto;margin-right: auto;width:350px;}

</style>
]]

title="Menu: Administrator Panel: Wizard: Set User"

-- The extra header we need ".header" to add.
response:include"/rtl/.header.lsp"
?>
<div id="help"></div>
<h1>Set User</h1>
<?lsp

local dio = ba.openio("fs") -- The drive I/O


-- Returns path1 + path2
function addPath(path1, path2)
   local path = path1.."/"..path2
   path=path:gsub("//","/") -- just in case
   return path
end

-- Attempts to create 'path1 + path2' and returns true if this is allowed
function canMakeDir(path1, path2)
   local path = addPath(path1, path2)
   if dio:stat(path) then -- Directory exist so assume access right is OK
      return true
   end
   if dio:mkdir(path) then
      dio:rmdir(path) -- Cleanup
      return true
   end
   return false
end


-- We try to suggest a path that is closer to the top of the 'drive' I/O
-- for the user's home directory. This makes it easier for users
-- to use the web-file-manager. This function tries to create a
-- directory at the top and keeps on iterating the bd.basepath
-- directory until we are allowed to create a directory.
function getsuggestedPath()
   local path = "/"
   local udir = "bdusers"
   if canMakeDir(path,udir) then
      return addPath(path,udir)
   end
   if bd.basepath then -- If we have the base path
      for n in string.gmatch(bd.basepath, "[^/]+") do
         path = addPath(path,n)
         if canMakeDir(path,udir) then
            return addPath(path,udir)
         end
      end
   end
   return addPath(path,udir) --Give up
end

-- split string into its components and create each directory.
-- Ignore all errors, except for last directory, which should not fail.
function mkpath(path)
   local e
   local npath="";
   for n in string.gmatch(path, "[^/]+") do
      npath = addPath(npath,n)
      local _
      _,e = dio:mkdir(npath)
   end
   if e then
      return "Cannot create \""..path.."\""
   end
   return nil
end

function trimString(s)
   return string.match(s, "^%s*(.-)%s*$")
end

-- If we have 'user', 'password', and 'path' url-encoded data and if
-- data is non empty.
function validPostData(data)
   if data.user and data.password and data.path then
      data.user = trimString(bd.xssfilter(data.user))
      data.password = trimString(data.password)
      data.path = trimString(data.path)
      if #data.user ~= 0 and #data.password ~= 0 and #data.path ~= 0 then
         return true
      end
      return false, "Invalid values"
   end
   return false
end

local udb,error = bd.db.user.load()

local userList={}
if udb and udb.v then
   for name,_ in pairs(udb.v) do
      table.insert(userList,name)
   end
end

local data = request:data();
local sugPath=nil -- Suggested base path for the users.
local isOk
isOk, error = validPostData(data)
if isOk then
   data.path = data.path:gsub("\\","/") -- In case user used Windogs slash
   local pn = addPath(data.path,data.user) -- home dir = Path + Name

   -- create home directory if it does not exist.
   if not dio:stat(pn) then
      if not dio:stat(data.path) then
         error = mkpath(data.path)
      end
      if not error then
         sugPath=data.path
         if not dio:mkdir(pn) then
            error = "Cannot create "..pn
         end
      end
   else
      sugPath=data.path
   end

   -- Create the /base/common directory if it does not exists
   if not error then
      dio:mkdir(addPath(data.path, "common"))
   end

   -- Load user database and set the new user.
   if not error then
      if udb and udb.v then
         local user={
            pwd=data.password,
            maxUsers=3,
            recycle=true,
            inactive=1200,
            roles={data.user, "users"},
            name=data.user,
            t=os.time(),
            info="User created by the wizard"
         }
         udb.v[data.user] = user -- Save in user database
         isOk, error = bd.db.user.save(udb)
      else
         error = "Cannot load and decode user database"
      end
   end

   -- Load constraints database and create a constraint that allows the user
   -- full access to his home directory.
   if not error then
      local cdb,error = bd.db.drvcnstr.load()
      if cdb and cdb.v then
         -- allow the user full access to his home directory
         local constraint={
            urls={pn.."/*"},
            methods={"GET","POST","PUT","PROPFIND","PROPPATCH",
               "MKCOL","DELETE","MOVE","COPY"},
            roles={data.user}}
         cdb.v[data.user] = constraint

         -- Create a common constraint for all users to /base/common
         if not cdb.v.common then
            cdb.v.common={
               urls={addPath(data.path,"common").."/*"},
               methods=constraint.methods,
               roles={"users"}}
         end
         -- Create a read only constraint for all users to /base
         if not cdb.v.users_r then
            cdb.v.users_r={
               urls={addPath(data.path,"*")},
               methods={"GET","PROPFIND"},
               roles={"users"}}
         end
         -- Create a read/write constraint available to 'admin'
         if not cdb.v.users_rw then
            cdb.v.users_rw={
               urls={addPath(data.path,"*")},
               methods=constraint.methods,
               roles={"admin"}}
         end
         ok,error = bd.db.drvcnstr.save(cdb)
      else
         error = "Cannot load and decode constraints database: "..error
      end
   end
   if not error then
       -- Load the modified user and constraints db
      local s = request:session()
      s.username=data.user
      s.password=data.password
      s.homedir=pn
      bd.db.roles.add(
        s.username,
        string.format("This role, which was created by the wizard, should only be assigned to the user '%s'.",
                      s.username))
      response:include"add2tunnel.lsp"
      return
   end -- if not error
else if not error then -- validPostData
?>

<table><tr>
<td>
<img src="../img/people.png" style="margin-right:2em"/>
</td>
<td>
<p><b>The simplified ACL manager allows you to do the following:</b></p>
<ul>
<li>Create a user and add the user to the user database.</li>
<li>Create a constraint that limits the user to his/her home directory.</li>
<li>Create the user's home directory on your hard drive.</li>
<li>Optionally give the user access to the HTTPS tunnel.</li>
</ul>

<?lsp
if #userList == 1 then
?>
<p style="color:red;font-weight:bold">You have not added any users besides the administrator.<br/>We recommend taking the time to read how the simplified ACL manager adds users to FuguHub. Move the mouse cursor over the help buttons below and carefully read the documentation.</p>
<?lsp
else
   print("<p><b>Users added:</b></p>")
   if #userList <= 5 then
      response:write("<ul>");
      for _, name in ipairs(userList) do
         response:write("<li>",name,"</li>");
      end
      response:write("</ul>");
   else
      local next=false
      for _, name in ipairs(userList) do
         if(next) then
            response:write(", ")
         else
            next=true
         end
         response:write(name)
      end
   end
?>
<?lsp
end
?>
</td>
</tr></table>
<?lsp
   end
end -- validPostData

if error then
   print("<p style='color:red'>The operation failed: ",bd.xssfilter(error), "</p>")
end
if not sugPath then
   sugPath = getsuggestedPath()
end
?>
<form method="post">
  <table class="roundedbox" id="UserInfo">
    <tr>
      <td>User&nbsp;name:</td>
      <td><input type='text' name='user'/></td>
      <td><img hlp='username' src='/rtl/bindows/html/images/questionmark.16.gif' alt='help'/></td>
    </tr>
    <tr>
      <td>Password:</td>
      <td><input type='text' name='password'/></td>
      <td><img hlp='password' src='/rtl/bindows/html/images/questionmark.16.gif' alt='help'/></td>
    </tr>
    <tr>
      <td>Base path</td>
      <td><input type="text" name="path" value="<?lsp=sugPath?>"/></td>
      <td><img hlp='basepath' src='/rtl/bindows/html/images/questionmark.16.gif' alt='help'/></td>
    </tr>
    <tr>
      <td>
      </td>
      <td>
        <input type="submit" value="Add User" style="float:right;width:100px;padding:2px;margin:5px"/>
      </td>
      <td>
        <img hlp='save' src='/rtl/bindows/html/images/questionmark.16.gif' alt='help'/>
      </td>
    </tr>

  </table>
</form> 
<p><br/></p>
<?lsp
response:include"../.footer.lsp"
?>
